#include <stdlib.h>
#include <string.h>
#include <rfcnb.h>
#include <smblib.h>
#include <tcp.h>

int verbose = 0;

char data[65536] = "Now is the time for all birds to roost.";

void usage (void)
{
  fprintf (stderr,
           "test_smblib [-s <file-service>] [-w <write-size] "
           "[-u <user>] [-b /\\] server\n");
  exit (1);
}

char *SMB_Prots[] = {
     "PC NETWORK PROGRAM 1.0",
     "MICROSOFT NETWORKS 1.03",
     "MICROSOFT NETWORKS 3.0",
     "DOS LANMAN1.0",
     "LANMAN1.0",
     "DOS LM1.2X002",
     "LM1.2X002",
     "DOS LANMAN2.1",
     "LANMAN2.1",
     "NT LM 0.12",
     NULL
   };

void do_error (int error, const char *fmt, ...)
{
  char err_str[1024];

  if (error == SMBlibE_Remote)
  {
    error = SMB_Get_Last_SMB_Err();
    SMB_Get_SMB_Error_Msg (SMBlib_Error_Class (error),
                           SMBlib_Error_Code (error),
                           err_str, sizeof(err_str)-1);
  }
  else
    SMB_Get_Error_Msg (error, err_str, sizeof(err_str)-1);

  if (fmt)
     vprintf (fmt, (&fmt)+1);
  printf ("  %s\n", err_str);
}

int main (int argc, char **argv)
{
  struct SMB_Tree    *tree;
  struct SMB_File    *file;
  struct SMB_Connect *con;
  int    opt, actual_size = 8192;
  int    i, written;
  char   server[80], service[80], service_name[160];
  char   password[80], username[80];
  char   file_name[80], slash = '\\';
  int    prot, lotc = FALSE;

  strcpy (service, "msfax");
  strcpy (username, "fred");

  while ((opt = getopt (argc, argv, "b:s:w:u:lv")) != EOF)
  {
    switch (opt)
    {
      case 's':
           strcpy (service, optarg);
           break;

      case 'w':
           actual_size = atoi (optarg) * 1024;
           break;

      case 'b':             /* set the slash character */
           slash = *optarg;
           break;

      case 'u':             /* Pick up the user name */
           strncpy (username, optarg, sizeof(username) - 1);
           break;

      case 'l':             /* do a logon and tree connect ... */
           lotc = TRUE;
           break;

      case 'v':             /* Set verbose flag */
           verbose++;
           dbug_init();
           break;

      default:
           usage();
    }
  }

  if (actual_size < 1024)
      actual_size = 1024;

  if (actual_size > sizeof(data))
      actual_size = sizeof(data);

  if (optind < argc)
  {
    strncpy (server, argv[optind], sizeof(server) - 1);
    optind++;

    if (optind < argc)
         strncpy (password, argv[optind], sizeof(password)-1);
    else strcpy (password, "nurk");
  }
  else
    strcpy (server, "nemesis");


  SMB_Init();
  sock_init();

  sprintf (service_name, "%c%c%s%c%s", slash, slash, server, slash, service);

  strcpy (file_name, "\\netfax.q");

  fprintf (stderr, "Connecting to %s ...\n", service_name);

  /*
   * Do the combined logon and tree connect
   */
  if (lotc)        
  {
    /* We must do a connect server and a negotiate first
     */
    con = SMB_Connect_Server (NULL, server);
    if (!con)
    {
      fprintf (stderr, "Unable to connect to server %s ...\n", server);
      do_error (SMB_Get_Last_Error(), NULL);
      exit (1);
    }

    /* We need to negotiate a protocol better than PC NetWork Program */

    if (SMB_Negotiate (con, SMB_Prots) < 0)
    {
      fprintf (stderr, "Unable to negotiate a protocol with server %s ...\n",
               server);
      do_error (SMB_Get_Last_Error(), NULL);
      exit (1);
    }

    if (verbose)
    {
      prot = SMB_Get_Protocol_IDX (con);
      fprintf (stderr, "The protocol negotiated was %i: %s\n",
               prot, SMB_Get_ProtocolStr(prot));
    }

    /* Should check the protocol level ... */

    if (SMB_Logon_TCon_Open (con, username, password, service_name, "A:",
                             &tree, file_name, SMB_AMODE_OPENRW, SMB_FA_ORD,
                             &file) < 0)
    {
      fprintf (stderr, "Unable to logon and tree connect to server %s ...\n",
               server);
      do_error (SMB_Get_Last_Error(), NULL);
      exit (1);                 /* Too bad ... */
    }
  }
  else
  {
    con = SMB_Connect (NULL, &tree, service_name, "Guest", password);
    if (!con)
    {                           
      fprintf (stderr, "Unable to connect ... \n");
      do_error (SMB_Get_Last_Error(), NULL);
      exit (1);
    }

    fprintf (stderr, "Attempting to open `%s' ...\n", file_name);

    file = SMB_Open (tree, NULL, file_name, SMB_AMODE_OPENRW, SMB_FA_ORD);
  }

  fprintf (stderr, "  The Max Buffer Size is %i\n", SMB_Get_Max_Buf_Siz(con));

  /*
   * Write, seek, read and close file test
   */
  if (file)
  {
    for (i = 0; i < actual_size; i++)
        data[i] = (char) (i & 0xFF);

    strncpy (data + actual_size - 20, "abcdefghijklmnopqrs", 19);

    fprintf (stderr, "Attempting to write %i bytes to the file ... \n",
             actual_size);

    written = SMB_Write (file, data, actual_size);
    if (written < 0)
    {
      do_error (SMB_Get_Last_Error(), "Write error:");
      exit (1);
    }

    fprintf (stderr, "Bytes written = %i\n", written);

    fprintf (stderr, "Now setting file pos to 512 and writing again ...\n");

    SMB_Lseek (file, 512, SEEK_SET);

    written = SMB_Write (file, data, actual_size);
    if (written < 0)
    {  
      do_error (SMB_Get_Last_Error(), "Write error:");
      exit (1);
    }

    fprintf (stderr, "Bytes written = %i\n", written);

    fprintf (stderr, "Now read from start for 1024 bytes ...\n");

    SMB_Lseek (file, 0, SEEK_SET);

    for (i = 0; i < sizeof(data); i++)
        data[i] = 0xff;

    SMB_Read (file, data, 1024);

    fprintf (stderr, "Attempting to close the file just opened ...\n");
    SMB_Close (file);
  }
  else
  {
    do_error (SMB_Get_Last_Error(), "Error opening file:");
    exit (1);
  }


  /*
   * Create, write and close file test
   */
  fprintf (stderr, "Attempting to create file 'xyzwertyaaa.dfgty' ...\n");

  file = SMB_Create (tree, NULL, "xyzwertyaaa.dfgty", SMB_FA_ORD);
  if (file)
  {
    fprintf (stderr, "File opened OK. Writing it ...\n");

    written = SMB_Write (file, data, actual_size);
    if (written < 0)
    {  
      do_error (SMB_Get_Last_Error(), "Error writing:");
      exit (1);
    }

    fprintf (stderr, "Bytes written: %i\n", written);
    fprintf (stderr, "Now close the file just created ...\n");
    SMB_Close (file);
  }


  /*
   * Delete file test
   */
  fprintf (stderr, "Attempting to delete the file 'delete.txt' ...\n");

  if (SMB_Delete (tree, "delete.txt", SMB_FA_ORD) < 0)
  {
    fprintf (stderr, "The file could not be deleted ... \n");
    do_error (SMB_Get_Last_Error(), NULL);

    fprintf (stderr, "Perhaps we are talking to PW4Ultrix ..., try again ...\n");
    fprintf (stderr, "Trying to delete \\DELETE.TXT ... \n");

    if (SMB_Delete (tree, "\\DELETE.TXT", SMB_FA_ORD) < 0)
    {
      fprintf (stderr, "Still could not be deleted ...\n");
      do_error (SMB_Get_Last_Error(),NULL);
    }
    else
      fprintf (stderr, "Deleted now ...\n");
  }
  else
    fprintf (stderr, "The file was deleted ... \n");


  /*
   * Long file name test
   */
  strcpy (file_name, "alongnamehurts.txt");

  fprintf (stderr, "Attempting to open '%s' ...\n", file_name);

  file = SMB_Create (tree, NULL, file_name, 0);
  if (!file)
     do_error (SMB_Get_Last_Error(), "Could not create file `%s'", file_name);


  /*
   * Create directory test
   */
  strncpy (file_name, "another-dir-name", sizeof(file_name)-1);

  fprintf (stderr, "Attempting to create directory `%s' ...\n", file_name);

  if (SMB_Create_Dir (tree, file_name) < 0)
     fprintf (stderr, "Creating directory `%s' failed ...\n", file_name);


  fprintf (stderr, "Attempting to check the directory `%s' ...\n", file_name);

  if (SMB_Check_Dir (tree, file_name) < 0)
     do_error (SMB_Get_Last_Error(), "Could not check directory `%s'", file_name);

  fprintf (stderr, "Attempting to delete directory `%s' ... \n", file_name);

  if (SMB_Delete_Dir (tree, file_name) < 0)
     do_error (SMB_Get_Last_Error(), "failed ...\n");

  fprintf (stderr, "Done\n");
  return (0);
}
