/*
 * Test password changing ...
 */

#include <stdlib.h>
#include <string.h>
#include <rfcnb.h>
#include <smblib.h>
#include <tcp.h>

char data[65536] = "Now is the time for all birds to roost.";
char slosh[2]    = "\\";

int verbose = FALSE;
int lotc    = FALSE;

char *SMB_Prots[] = {
     "PC NETWORK PROGRAM 1.0",
     "MICROSOFT NETWORKS 1.03",
     "MICROSOFT NETWORKS 3.0",
     "LANMAN1.0",
     "LM1.2X002",
     "LANMAN2.1",
     "NT LM 0.12",
     "NT LANMAN 1.0",
     NULL
   };

void usage (char *prog)
{
  fprintf (stderr, "Usage: %s -u <user> -p <password> -o <oldpass> "
           "server\n", prog);
}

void do_error (int error)
{
  char err_str[1024];

  if (error == SMBlibE_Remote)
  {
    error = SMB_Get_Last_SMB_Err();
    SMB_Get_SMB_Error_Msg (SMBlib_Error_Class (error),
                           SMBlib_Error_Code (error),
                           err_str, sizeof(err_str)-1);
  }
  else
    SMB_Get_Error_Msg (error, err_str, sizeof(err_str)-1);

  printf ("  %s\n", err_str);
}

int main (int argc, char **argv)
{
  struct SMB_Connect *con;
  struct SMB_Tree    *tree;
  int    opt, status;
  char   server[80], service[80], service_name[160], username[80];
  char   password[80], old_password[80];

  server[0] = 0;
  strncpy (service, "IPC$", sizeof(service)-1);
  service_name[0] = 0;
  username[0] = 0;
  password[0] = 0;
  old_password[0] = 0;

  while ((opt = getopt (argc, argv, "s:u:o:p:v")) != EOF)
  {
    switch (opt)
    {
      case 's':
           strcpy (service, optarg);
           break;

      case 'u':             /* Pick up the user name */
           strncpy (username, optarg, sizeof (username) - 1);
           break;

      case 'l':             /* do a logon and tree connect ... */
           lotc = TRUE;
           break;

      case 'v':             /* Verbose? */
           verbose = TRUE;
           dbug_init ();
           break;

      case 'p':             /* Pick up the password */
           strncpy (password, optarg, sizeof (password) - 1);
           break;

      case 'o':             /* Pick up old password */
           strncpy (old_password, optarg, sizeof (old_password) - 1);
           break;

      default:
           usage (argv[0]);
           exit (1);
           break;
    }
  }

  if (optind < argc)
  {                            /* Some more parameters, assume is the server */
    strncpy (server, argv[optind], sizeof (server) - 1);
    optind++;
  }
  else
    strcpy (server, "nemesis");

  if (verbose == TRUE)
  { 
    fprintf (stderr, "Changing password for User: %s, on server: %s\n", username, server);
    fprintf (stderr, "         from: %s, to: %s\n", old_password, password);
  }

  SMB_Init();
  sock_init();


  /* We connect to the server and negotiate
   */
  con = SMB_Connect_Server (NULL, server);
  if (!con)
  {    
    fprintf (stderr, "Unable to connect to server %s ...\n", server);
    do_error (SMB_Get_Last_Error());
    exit (1);
  }

  /* We need to negotiate a protocol better than PC NetWork Program
   */

  if (SMB_Negotiate (con, SMB_Prots) < 0)
  { 
    fprintf (stderr, "Unable to negotiate a protocol with server %s ...\n",
             server);
    do_error (SMB_Get_Last_Error());
    exit (1);
  }

  sprintf (service_name, "\\\\%s\\%s", server, service); /* Could blow up */

  tree = SMB_Logon_And_TCon (con, NULL, username, old_password,
                             service_name, "?????");
  if (!tree)
  {
    fprintf (stderr, "Unable to logon and tree connect to server %s ...\n",
             server);
    do_error (SMB_Get_Last_Error());
    exit (1);
  }

  /* OK, now we have a tree ... Issue the API call ...
   */
  if (SMBapi_NetUserPasswordSet (tree, username, old_password, password, &status) < 0)
  {
    fprintf (stderr, "Unable to change password on server %s ...\n", server);
    do_error (SMB_Get_Last_Error());
    exit (1);
  }

  if (status)
       fprintf (stderr, "Password change failed. Status is: %i\n", status);
  else fprintf (stderr, "Password change succeeded.\n");

  return (0);
}
